import requests
import hmac
import hashlib
import time
import sys
import os
import pandas as pd
from datetime import datetime
import json
from time import sleep
from dotenv import load_dotenv
from tqdm import tqdm

load_dotenv()
API_KEY = os.getenv('cryptocom_api_key')
SECRET_KEY = os.getenv('cryptocom_api_secret')

MAX_LEVEL = 3

def params_to_str(obj, level):
    if level >= MAX_LEVEL:
        return str(obj)

    return_str = ""
    for key in sorted(obj):
        return_str += key
        if obj[key] is None:
            return_str += 'null'
        elif isinstance(obj[key], list):
            for subObj in obj[key]:
                return_str += params_to_str(subObj, level+1)
        else:
            return_str += str(obj[key])
    return return_str


def sign_request(req):
	param_str = ""
	if "params" in req:
		param_str = params_to_str(req['params'], 0)
	
	payload_str = req['method'] + str(req['id']) + req['api_key'] + param_str + str(req['nonce'])
	req['sig'] = hmac.new(
		bytes(str(SECRET_KEY), 'utf-8'),
		msg=bytes(payload_str, 'utf-8'),
		digestmod=hashlib.sha256
	).hexdigest()
	return req

def get_account_summary():
    req = {
        "id": 11,
        "method": "private/get-account-summary",
        "params": {},
        "api_key": API_KEY,
        "nonce": int(time.time() * 1000)
    }
    signed_req = sign_request(req)
    response = requests.post("https://api.crypto.com/v2/private/get-account-summary", json=signed_req)
    return response.json()

def account_summary_to_df(data):
    if 'result' in data and 'accounts' in data['result']:
        df = pd.json_normalize(data['result']['accounts'])
        df['timestamp'] = datetime.now()
        df['balance'] = df['balance'].map('{:.8f}'.format)
        df['available'] = df['available'].map('{:.8f}'.format)
        df['order'] = df['order'].map('{:.8f}'.format)
        return df
    else:
        print(f"Unexpected response: {data}")
        return pd.DataFrame()

def get_account_summary_df():
    data = get_account_summary()
    return account_summary_to_df(data)
